# -*- coding: utf-8 -*-
# pylint: disable=too-many-lines
from django import get_version
from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.db.models.query import QuerySet
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from jsonfield.fields import JSONField
from model_utils import Choices
from packaging.version import (
    parse as parse_version,  # pylint: disable=no-name-in-module,import-error
)
from swapper import load_model
from swapper import swappable_setting

from notifications import settings as notifications_settings
from notifications.signals import notify
from notifications.utils import id2slug

if parse_version(get_version()) >= parse_version('1.8.0'):
    from django.contrib.contenttypes.fields import GenericForeignKey  # noqa
else:
    from django.contrib.contenttypes.generic import GenericForeignKey  # noqa


def is_soft_delete():
    return notifications_settings.get_config()['SOFT_DELETE']


def assert_soft_delete():
    if not is_soft_delete():
        # msg = """To use 'deleted' field, please set 'SOFT_DELETE'=True in settings.
        # Otherwise NotificationQuerySet.unread and NotificationQuerySet.read do NOT filter by 'deleted' field.
        # """
        msg = 'REVERTME'
        raise ImproperlyConfigured(msg)


class NotificationQuerySet(models.query.QuerySet):
    ''' Notification QuerySet '''
    def unsent(self):
        return self.filter(emailed=False)

    def sent(self):
        return self.filter(emailed=True)

    def unread(self, include_deleted=False):
        """Return only unread items in the current queryset"""
        if is_soft_delete() and not include_deleted:
            return self.filter(unread=True, deleted=False)

        # When SOFT_DELETE=False, developers are supposed NOT to touch 'deleted' field.
        # In this case, to improve query performance, don't filter by 'deleted' field
        return self.filter(unread=True)

    def read(self, include_deleted=False):
        """Return only read items in the current queryset"""
        if is_soft_delete() and not include_deleted:
            return self.filter(unread=False, deleted=False)

        # When SOFT_DELETE=False, developers are supposed NOT to touch 'deleted' field.
        # In this case, to improve query performance, don't filter by 'deleted' field
        return self.filter(unread=False)

    def mark_all_as_read(self, recipient=None):
        """Mark as read any unread messages in the current queryset.

        Optionally, filter these by recipient first.
        """
        # We want to filter out read ones, as later we will store
        # the time they were marked as read.
        qset = self.unread(True)
        if recipient:
            qset = qset.filter(recipient=recipient)

        return qset.update(unread=False)

    def mark_all_as_unread(self, recipient=None):
        """Mark as unread any read messages in the current queryset.

        Optionally, filter these by recipient first.
        """
        qset = self.read(True)

        if recipient:
            qset = qset.filter(recipient=recipient)

        return qset.update(unread=True)

    def deleted(self):
        """Return only deleted items in the current queryset"""
        assert_soft_delete()
        return self.filter(deleted=True)

    def active(self):
        """Return only active(un-deleted) items in the current queryset"""
        assert_soft_delete()
        return self.filter(deleted=False)

    def mark_all_as_deleted(self, recipient=None):
        """Mark current queryset as deleted.
        Optionally, filter by recipient first.
        """
        assert_soft_delete()
        qset = self.active()
        if recipient:
            qset = qset.filter(recipient=recipient)

        return qset.update(deleted=True)

    def mark_all_as_active(self, recipient=None):
        """Mark current queryset as active(un-deleted).
        Optionally, filter by recipient first.
        """
        assert_soft_delete()
        qset = self.deleted()
        if recipient:
            qset = qset.filter(recipient=recipient)

        return qset.update(deleted=False)

    def mark_as_unsent(self, recipient=None):
        qset = self.sent()
        if recipient:
            qset = qset.filter(recipient=recipient)
        return qset.update(emailed=False)

    def mark_as_sent(self, recipient=None):
        qset = self.unsent()
        if recipient:
            qset = qset.filter(recipient=recipient)
        return qset.update(emailed=True)


class AbstractNotification(models.Model):
    """
    Action model describing the actor acting out a verb (on an optional
    target).
    Nomenclature based on http://activitystrea.ms/specs/atom/1.0/

    Generalized Format::

        <actor> <verb> <time>
        <actor> <verb> <target> <time>
        <actor> <verb> <action_object> <target> <time>

    Examples::

        <justquick> <reached level 60> <1 minute ago>
        <brosner> <commented on> <pinax/pinax> <2 hours ago>
        <washingtontimes> <started follow> <justquick> <8 minutes ago>
        <mitsuhiko> <closed> <issue 70> on <mitsuhiko/flask> <about 2 hours ago>

    Unicode Representation::

        justquick reached level 60 1 minute ago
        mitsuhiko closed issue 70 on mitsuhiko/flask 3 hours ago

    HTML Representation::

        <a href="http://oebfare.com/">brosner</a> commented on <a href="http://github.com/pinax/pinax">pinax/pinax</a> 2 hours ago # noqa

    """
    LEVELS = Choices('success', 'info', 'warning', 'error')
    level = models.CharField(_('level'), choices=LEVELS, default=LEVELS.info, max_length=20)

    recipient = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='notifications',
        verbose_name=_('recipient'),
        blank=False,
    )
    unread = models.BooleanField(_('unread'), default=True, blank=False, db_index=True)

    actor_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        related_name='notify_actor',
        verbose_name=_('actor content type')
    )
    actor_object_id = models.CharField(_('actor object id'), max_length=255)
    actor = GenericForeignKey('actor_content_type', 'actor_object_id')
    actor.short_description = _('actor')

    verb = models.CharField(_('verb'), max_length=255)
    description = models.TextField(_('description'), blank=True, null=True)

    target_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        related_name='notify_target',
        verbose_name=_('target content type'),
        blank=True,
        null=True
    )
    target_object_id = models.CharField(_('target object id'), max_length=255, blank=True, null=True)
    target = GenericForeignKey('target_content_type', 'target_object_id')
    target.short_description = _('target')

    action_object_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        related_name='notify_action_object',
        verbose_name=_('action object content type'),
        blank=True,
        null=True
    )
    action_object_object_id = models.CharField(_('action object object id'), max_length=255, blank=True, null=True)
    action_object = GenericForeignKey('action_object_content_type', 'action_object_object_id')
    action_object.short_description = _('action object')

    timestamp = models.DateTimeField(_('timestamp'), default=timezone.now, db_index=True)

    public = models.BooleanField(_('public'), default=True, db_index=True)
    deleted = models.BooleanField(_('deleted'), default=False, db_index=True)
    emailed = models.BooleanField(_('emailed'), default=False, db_index=True)

    data = JSONField(_('data'), blank=True, null=True)

    objects = NotificationQuerySet.as_manager()

    class Meta:
        abstract = True
        ordering = ('-timestamp',)
        # speed up notifications count query
        indexes = [
            models.Index(fields=['recipient', 'unread']),
        ]
        verbose_name = _('Notification')
        verbose_name_plural = _('Notifications')

    def __str__(self):
        ctx = {
            'actor': self.actor,
            'verb': self.verb,
            'action_object': self.action_object,
            'target': self.target,
            'timesince': self.timesince()
        }
        if self.target:
            if self.action_object:
                return _('%(actor)s %(verb)s %(action_object)s on %(target)s %(timesince)s ago') % ctx
            return _('%(actor)s %(verb)s %(target)s %(timesince)s ago') % ctx
        if self.action_object:
            return _('%(actor)s %(verb)s %(action_object)s %(timesince)s ago') % ctx
        return _('%(actor)s %(verb)s %(timesince)s ago') % ctx

    def timesince(self, now=None):
        """
        Shortcut for the ``django.utils.timesince.timesince`` function of the
        current timestamp.
        """
        from django.utils.timesince import timesince as timesince_
        return timesince_(self.timestamp, now)

    @property
    def slug(self):
        return id2slug(self.id)

    def mark_as_read(self):
        if self.unread:
            self.unread = False
            self.save()

    def mark_as_unread(self):
        if not self.unread:
            self.unread = True
            self.save()

    # def actor_object_url(self):
    #     try:
    #         url = reverse("admin:{0}_{1}_change".format(self.actor_content_type.app_label,
    #                                                     self.actor_content_type.model),
    #                       args=(self.actor_object_id,))
    #         return format_html("<a href='{url}'>{id}</a>", url=url, id=self.actor_object_id)
    #     except NoReverseMatch:
    #         return self.actor_object_id
    #
    # def action_object_url(self):
    #     try:
    #         url = reverse("admin:{0}_{1}_change".format(self.action_object_content_type.app_label,
    #                                                     self.action_content_type.model),
    #                       args=(self.action_object_id,))
    #         return format_html("<a href='{url}'>{id}</a>", url=url, id=self.action_object_object_id)
    #     except NoReverseMatch:
    #         return self.action_object_object_id
    #
    # def target_object_url(self):
    #     try:
    #         url = reverse("admin:{0}_{1}_change".format(self.target_content_type.app_label,
    #                                                     self.target_content_type.model),
    #                       args=(self.target_object_id,))
    #         return format_html("<a href='{url}'>{id}</a>", url=url, id=self.target_object_id)
    #     except NoReverseMatch:
    #         return self.target_object_id

    def naturalday(self):
        """
        Shortcut for the ``humanize``.
        Take a parameter humanize_type. This parameter control the which humanize method use.
        Return ``today``, ``yesterday`` ,``now``, ``2 seconds ago``etc.
        """
        from django.contrib.humanize.templatetags.humanize import naturalday
        return naturalday(self.timestamp)

    def naturaltime(self):
        from django.contrib.humanize.templatetags.humanize import naturaltime
        return naturaltime(self.timestamp)


def notify_handler(verb, **kwargs):
    """
    Handler function to create Notification instance upon action signal call.
    """
    # Pull the options out of kwargs
    Notification = load_model('notifications', 'Notification')

    kwargs.pop('signal', None)
    recipient = kwargs.pop('recipient')
    actor = kwargs.pop('sender')
    optional_objs = [
        (kwargs.pop(opt, None), opt)
        for opt in ('target', 'action_object')
    ]
    public = bool(kwargs.pop('public', True))
    description = kwargs.pop('description', None)
    timestamp = kwargs.pop('timestamp', timezone.now())
    level = kwargs.pop('level', Notification.LEVELS.info)
    actor_for_concrete_model = kwargs.pop('actor_for_concrete_model', True)

    # Check if User or Group
    if isinstance(recipient, Group):
        recipients = recipient.user_set.all()
    elif isinstance(recipient, (QuerySet, list)):
        recipients = recipient
    else:
        recipients = [recipient]

    new_notifications = []

    for recipient in recipients:
        newnotify = Notification(
            recipient=recipient,
            actor_content_type=ContentType.objects.get_for_model(actor, for_concrete_model=actor_for_concrete_model),
            actor_object_id=actor.pk,
            verb=str(verb),
            public=public,
            description=description,
            timestamp=timestamp,
            level=level,
        )

        # Set optional objects
        for obj, opt in optional_objs:
            if obj is not None:
                for_concrete_model = kwargs.pop(f'{opt}_for_concrete_model', True)
                setattr(newnotify, '%s_object_id' % opt, obj.pk)
                setattr(newnotify, '%s_content_type' % opt,
                        ContentType.objects.get_for_model(obj, for_concrete_model=for_concrete_model))

        if kwargs:
            # set kwargs as model column if available
            for key in list(kwargs.keys()):
                if hasattr(Notification, key):
                    setattr(newnotify, key, kwargs.pop(key))
            newnotify.data = kwargs

        newnotify.save()
        new_notifications.append(newnotify)

    # TODO: Send Django Channels WebSocket Signal here

    return new_notifications


class Notification(AbstractNotification):

    class Meta(AbstractNotification.Meta):
        abstract = False
        swappable = swappable_setting('notifications', 'Notification')


# connect the signal
notify.connect(notify_handler, dispatch_uid='notifications.models.notification')




from rest_framework.routers import DefaultRouter

from .views import NotificationViewSet

router = DefaultRouter()
router.register("notifications", NotificationViewSet, basename='notification')

urlpatterns = router.urls

app_name = 'notifications'



'''' Django notifications utils file '''
# -*- coding: utf-8 -*-
import sys


if sys.version > '3':
    long = int  # pylint: disable=invalid-name


def slug2id(slug):
    return long(slug) - 110909


def id2slug(notification_id):
    return notification_id + 110909


def get_content_type_key(content_type):
    return f"{content_type.app_label.lower()}_{content_type.model.lower()}"



from django.db import models


class Customer(models.Model):
    name = models.CharField(max_length=64)
    address = models.TextField()

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return f"foo/{self.id}/"


class TargetObject(Customer):
    def get_url_for_notifications(self, notification, request):
        return f"bar/{self.id}/"



import swapper
from django.contrib import admin
from notifications.admin import AbstractNotificationAdmin

Notification = swapper.load_model('notifications', 'Notification')


@admin.register(Notification)
class NotificationAdmin(AbstractNotificationAdmin):
    pass



from django.forms import model_to_dict
from notifications.utils import id2slug
from notifications.settings import get_config

def get_object_url(instance, notification, request):
    """
    Get url representing the instance object.
    This will return instance.get_url_for_notifications()
    with parameters `notification` and `request`,
    if it is defined and get_absolute_url() otherwise
    """
    if hasattr(instance, 'get_url_for_notifications'):
        return instance.get_url_for_notifications(notification, request)
    elif hasattr(instance, 'get_absolute_url'):
        return instance.get_absolute_url()
    return None

def get_num_to_fetch(request):
    default_num_to_fetch = get_config()['NUM_TO_FETCH']
    try:
        # If they don't specify, make it 5.
        num_to_fetch = request.GET.get('max', default_num_to_fetch)
        num_to_fetch = int(num_to_fetch)
        if not (1 <= num_to_fetch <= 100):
            num_to_fetch = default_num_to_fetch
    except ValueError:  # If casting to an int fails.
        num_to_fetch = default_num_to_fetch
    return num_to_fetch

def get_notification_list(request, method_name='all'):
    num_to_fetch = get_num_to_fetch(request)
    notification_list = []
    for notification in getattr(request.user.notifications, method_name)()[0:num_to_fetch]:
        struct = model_to_dict(notification)
        struct['slug'] = id2slug(notification.id)
        if notification.actor:
            struct['actor'] = str(notification.actor)
            actor_url = get_object_url(
                notification.actor, notification, request)
            if actor_url:
                struct['actor_url'] = actor_url
        if notification.target:
            struct['target'] = str(notification.target)
            target_url = get_object_url(
                notification.target, notification, request)
            if target_url:
                struct['target_url'] = target_url
        if notification.action_object:
            struct['action_object'] = str(notification.action_object)
            action_object_url = get_object_url(
                notification.action_object, notification, request)
            if action_object_url:
                struct['action_object_url'] = action_object_url
        if notification.data:
            struct['data'] = notification.data
        notification_list.append(struct)
        if request.GET.get('mark_as_read'):
            notification.mark_as_read()
    return notification_list



# -*- coding: utf-8 -*-
"""
    django-notifications
    ~~~~~
    A GitHub notification alike app for Django.
    :copyright: (c) 2015 by django-notifications team.
    :license: BSD, see LICENSE.txt for more details.
"""

# PEP 386-compliant version number: N.N[.N]+[{a|b|c|rc}N[.N]+][.postN][.devN]
__version__ = '1.0.0'

default_app_config = 'notifications.apps.Config'  # pylint: disable=invalid-name



''' Django notifications settings file '''
# -*- coding: utf-8 -*-
from django.conf import settings


CONFIG_DEFAULTS = {
    'SOFT_DELETE': False,
    'NUM_TO_FETCH': 10,
    'CACHE_TIMEOUT': 2,
    'NOTIFICATIONS_SERIALIZER': 'notifications.serializers.NotificationSerializer',
    'ACTOR_OBJECT_IN_RESPONSE': False,
    'ACTION_OBJECT_IN_RESPONSE': False,
    'TARGET_OBJECT_IN_RESPONSE': False,
}


def get_config():
    user_config = getattr(settings, 'NOTIFICATIONS_CONFIG', {})

    config = CONFIG_DEFAULTS.copy()
    config.update(user_config)

    return config



from notifications.apps import Config as NotificationConfig


class SampleNotificationsConfig(NotificationConfig):
    name = 'notifications.tests.sample_notifications'
    label = 'sample_notifications'



''' Django notification urls for tests '''
# -*- coding: utf-8 -*-
from django import get_version
from django.contrib import admin
from packaging.version import (
    parse as parse_version,  # pylint: disable=no-name-in-module,import-error
)

from notifications.tests.views import (
    live_tester,  # pylint: disable=no-name-in-module,import-error
)
from notifications.tests.views import make_notification

if parse_version(get_version()) >= parse_version('2.1'):
    from django.contrib.auth.views import LoginView
    from django.urls import include, path  # noqa
    urlpatterns = [
        path('test_make/', make_notification),
        path('test/', live_tester),
        path('login/', LoginView.as_view(), name='login'),  # reverse for django login is not working
        path('admin/', admin.site.urls),
        path('', include('notifications.urls', namespace='notifications')),
    ]
elif parse_version(get_version()) >= parse_version('2.0') and parse_version(get_version()) < parse_version('2.1'):
    from django.contrib.auth.views import login
    from django.urls import include, path  # noqa
    urlpatterns = [
        path('test_make/', make_notification),
        path('test/', live_tester),
        path('login/', login, name='login'),  # reverse for django login is not working
        path('admin/', admin.site.urls),
        path('', include('notifications.urls', namespace='notifications')),
    ]
else:
    from django.conf.urls import include, url
    from django.contrib.auth.views import login
    urlpatterns = [
        url(r'^login/$', login, name='login'),  # reverse for django login is not working
        url(r'^test_make/', make_notification),
        url(r'^test/', live_tester),
        url(r'^', include('notifications.urls', namespace='notifications')),
        url(r'^admin/', admin.site.urls),
    ]



from rest_framework import serializers
from swapper import load_model

from .settings import get_config
from .utils import get_content_type_key

Notification = load_model('notifications', 'Notification')


def serialize_object(_object):
    class ObjectSerializer(serializers.ModelSerializer):
        class Meta:
            model = _object.__class__
            fields = '__all__'

    return ObjectSerializer(_object).data


class NotificationSerializer(serializers.ModelSerializer):
    action_object_key = serializers.SerializerMethodField()
    target_key = serializers.SerializerMethodField()
    actor_key = serializers.SerializerMethodField()

    if get_config()["ACTOR_OBJECT_IN_RESPONSE"]:
        actor = serializers.SerializerMethodField()

    if get_config()["ACTION_OBJECT_IN_RESPONSE"]:
        action_object = serializers.SerializerMethodField()

    if get_config()["TARGET_OBJECT_IN_RESPONSE"]:
        target = serializers.SerializerMethodField()

    class Meta:
        model = Notification
        fields = '__all__'

    def get_action_object(self, obj):
        if obj.action_object:
            return serialize_object(obj.action_object)
        return None

    def get_action_object_key(self, obj):
        if content_type := obj.action_object_content_type:
            return get_content_type_key(content_type)
        return None

    def get_target(self, obj):
        if obj.target:
            return serialize_object(obj.target)
        return None

    def get_target_key(self, obj):
        if content_type := obj.target_content_type:
            return get_content_type_key(content_type)
        return None

    def get_actor(self, obj):
        if obj.actor:
            return serialize_object(obj.actor)
        return None

    def get_actor_key(self, obj):
        if content_type := obj.actor_content_type:
            return get_content_type_key(content_type)
        return None



# Generated by Django 3.0.5 on 2020-04-11 12:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('level', models.CharField(choices=[('success', 'success'), ('info', 'info'), ('warning', 'warning'), ('error', 'error')], default='info', max_length=20)),
                ('unread', models.BooleanField(db_index=True, default=True)),
                ('actor_object_id', models.CharField(max_length=255)),
                ('verb', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
                ('target_object_id', models.CharField(blank=True, max_length=255, null=True)),
                ('action_object_object_id', models.CharField(blank=True, max_length=255, null=True)),
                ('timestamp', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('public', models.BooleanField(db_index=True, default=True)),
                ('deleted', models.BooleanField(db_index=True, default=False)),
                ('emailed', models.BooleanField(db_index=True, default=False)),
                ('data', jsonfield.fields.JSONField(blank=True, null=True)),
                ('details', models.CharField(blank=True, max_length=64, null=True)),
                ('action_object_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_action_object', to='contenttypes.ContentType')),
                ('actor_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notify_actor', to='contenttypes.ContentType')),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
                ('target_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_target', to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('-timestamp',),
                'abstract': False,
                'index_together': {('recipient', 'unread')},
            },
        ),
    ]



# Generated by Django 4.2.14 on 2024-07-24 09:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Customer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=64)),
                ("address", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="TargetObject",
            fields=[
                (
                    "customer_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="test_models.customer",
                    ),
                ),
            ],
            bases=("test_models.customer",),
        ),
    ]



''' Django notifications template tags file '''
# -*- coding: utf-8 -*-
from django import get_version
from django.core.cache import cache
from django.template import Library
from django.utils.html import format_html
from packaging.version import (
    parse as parse_version,  # pylint: disable=no-name-in-module,import-error
)

from notifications import settings
from notifications.settings import get_config

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import (
        reverse,  # pylint: disable=no-name-in-module,import-error
    )

register = Library()


def get_cached_notification_unread_count(user):

    return cache.get_or_set(
        'cache_notification_unread_count',
        user.notifications.unread().count,
        settings.get_config()['CACHE_TIMEOUT']
    )

def notifications_unread(context):
    user = user_context(context)
    if not user:
        return ''
    return get_cached_notification_unread_count(user)


if parse_version(get_version()) >= parse_version('2.0'):
    notifications_unread = register.simple_tag(takes_context=True)(notifications_unread)  # pylint: disable=invalid-name
else:
    notifications_unread = register.assignment_tag(takes_context=True)(notifications_unread)  # noqa


@register.filter
def has_notification(user):
    if user:
        return user.notifications.unread().exists()
    return False


# Requires vanilla-js framework - http://vanilla-js.com/
@register.simple_tag
def register_notify_callbacks(badge_class='live_notify_badge',  # pylint: disable=too-many-arguments,missing-docstring
                              menu_class='live_notify_list',
                              refresh_period=15,
                              callbacks='',
                              api_name='list',
                              fetch=5,
                              nonce=None,
                              mark_as_read=False
                              ):
    refresh_period = int(refresh_period) * 1000

    if api_name == 'list':
        api_url = reverse('notifications:live_unread_notification_list')
    elif api_name == 'count':
        api_url = reverse('notifications:live_unread_notification_count')
    else:
        return ""
    definitions = """
        notify_badge_class='{badge_class}';
        notify_menu_class='{menu_class}';
        notify_api_url='{api_url}';
        notify_fetch_count='{fetch_count}';
        notify_unread_url='{unread_url}';
        notify_mark_all_unread_url='{mark_all_unread_url}';
        notify_refresh_period={refresh};
        notify_mark_as_read={mark_as_read};
    """.format(
        badge_class=badge_class,
        menu_class=menu_class,
        refresh=refresh_period,
        api_url=api_url,
        unread_url=reverse('notifications:unread'),
        mark_all_unread_url=reverse('notifications:mark_all_as_read'),
        fetch_count=fetch,
        mark_as_read=str(mark_as_read).lower()
    )

    # add a nonce value to the script tag if one is provided
    nonce_str = ' nonce="{nonce}"'.format(nonce=nonce) if nonce else ""

    script = '<script type="text/javascript"{nonce}>'.format(nonce=nonce_str) + definitions
    for callback in callbacks.split(','):
        script += "register_notifier(" + callback + ");"
    script += "</script>"
    return format_html(script)


@register.simple_tag(takes_context=True)
def live_notify_badge(context, badge_class='live_notify_badge'):
    user = user_context(context)
    if not user:
        return ''

    html = "<span class='{badge_class}'>{unread}</span>".format(
        badge_class=badge_class, unread=get_cached_notification_unread_count(user)
    )
    return format_html(html)


@register.simple_tag
def live_notify_list(list_class='live_notify_list'):
    html = "<ul class='{list_class}'></ul>".format(list_class=list_class)
    return format_html(html)


def user_context(context):
    if 'user' not in context:
        return None

    request = context['request']
    user = request.user
    try:
        user_is_anonymous = user.is_anonymous()
    except TypeError:  # Django >= 1.11
        user_is_anonymous = user.is_anonymous

    if user_is_anonymous:
        return None
    return user



#!/usr/bin/env python
# -*- coding: utf-8 -*-
''' Django notification setup file for pip package '''
import ast
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup  # pylint: disable=no-name-in-module,import-error


_version_re = re.compile(r'__version__\s+=\s+(.*)')  # pylint: disable=invalid-name

with open('notifications/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(  # pylint: disable=invalid-name
        f.read().decode('utf-8')).group(1)))


setup(
    name='drf-notifications',
    version=version,
    description='GitHub notifications alike app for Django.',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='django-notifications team',
    author_email='soomroaaqib55@gmail.com',
    url='https://github.com/soomroaaqib/drf-notifications.git',
    install_requires=[
        'django>=4.2',
        'django-model-utils>=3.1.0',
        'jsonfield>=2.1.0',
        'swapper',
        "packaging",
        "djangorestframework==3.16.0",
    ],
    test_requires=[
        'django>=3.2',
        'django-model-utils>=3.1.0',
        'jsonfield>=2.1.0',
    ],
    packages=[
        'notifications',
        'notifications.migrations',
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Framework :: Django :: 5.1',
        'Framework :: Django :: 5.2',
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Topic :: Utilities'
    ],
    keywords='django notifications github action event stream drf rest_framework api',
    license='BSD',
)



''' Django notifications apps file '''
# -*- coding: utf-8 -*-
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class Config(AppConfig):
    name = "notifications"
    verbose_name = _("Notifications")
    default_auto_field = 'django.db.models.AutoField'

    def ready(self):
        super(Config, self).ready()
        # this is for backwards compatibility
        import notifications.signals
        notifications.notify = notifications.signals.notify



# Generated by Django 5.2 on 2025-07-23 09:32

import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('level', models.CharField(choices=[('success', 'success'), ('info', 'info'), ('warning', 'warning'), ('error', 'error')], default='info', max_length=20, verbose_name='level')),
                ('unread', models.BooleanField(db_index=True, default=True, verbose_name='unread')),
                ('actor_object_id', models.CharField(max_length=255, verbose_name='actor object id')),
                ('verb', models.CharField(max_length=255, verbose_name='verb')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('target_object_id', models.CharField(blank=True, max_length=255, null=True, verbose_name='target object id')),
                ('action_object_object_id', models.CharField(blank=True, max_length=255, null=True, verbose_name='action object object id')),
                ('timestamp', models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='timestamp')),
                ('public', models.BooleanField(db_index=True, default=True, verbose_name='public')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='deleted')),
                ('emailed', models.BooleanField(db_index=True, default=False, verbose_name='emailed')),
                ('data', jsonfield.fields.JSONField(blank=True, null=True, verbose_name='data')),
                ('action_object_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_action_object', to='contenttypes.contenttype', verbose_name='action object content type')),
                ('actor_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notify_actor', to='contenttypes.contenttype', verbose_name='actor content type')),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL, verbose_name='recipient')),
                ('target_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_target', to='contenttypes.contenttype', verbose_name='target content type')),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
                'ordering': ('-timestamp',),
                'abstract': False,
                'swappable': 'NOTIFICATIONS_NOTIFICATION_MODEL',
                'indexes': [models.Index(fields=['recipient', 'unread'], name='notificatio_recipie_8bedf2_idx')],
            },
        ),
    ]



import os
from unittest import skipUnless

from django.contrib.auth.models import User
import swapper

from notifications.signals import notify
from notifications.tests.tests import AdminTest as BaseAdminTest
from notifications.tests.tests import NotificationTest as BaseNotificationTest

Notification = swapper.load_model('notifications', 'Notification')


@skipUnless(os.environ.get('SAMPLE_APP', False), 'Running tests on standard django-notifications models')
class AdminTest(BaseAdminTest):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        BaseAdminTest.app_name = 'sample_notifications'


@skipUnless(os.environ.get('SAMPLE_APP', False), 'Running tests on standard django-notifications models')
class NotificationTest(BaseNotificationTest):
    pass


class TestExtraDataCustomAccessor(NotificationTest):
    def setUp(self):
        self.from_user = User.objects.create_user(username="from_extra", password="pwd", email="example@example.com")
        self.to_user = User.objects.create_user(username="to_extra", password="pwd", email="example@example.com")
        notify.send(
            self.from_user,
            recipient=self.to_user,
            verb='commented',
            action_object=self.from_user,
            url="/learn/ask-a-pro/q/test-question-9/299/",
            other_content="Hello my 'world'",
            details="test detail"
        )

    def test_extra_data(self):
        notification = Notification.objects.get(details="test detail")
        assert notification, "Expected a notification retrieved by custom extra data accessor"
        assert notification.details == "test detail", "Custom accessor should return set value"
        assert "details" not in notification.data, "Custom accessor should not be in json data"



from django.utils.module_loading import import_string
from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import GenericViewSet
from swapper import load_model

from notifications import settings as notification_settings

Notification = load_model('notifications', 'Notification')

class NotificationViewSet(
    mixins.ListModelMixin,
    GenericViewSet,
):
    permission_classes = [IsAuthenticated]

    serializer_class = None
    _serializer_class = notification_settings.get_config()["NOTIFICATIONS_SERIALIZER"]

    def get_serializer_class(self):
        """
        If serializer_class is set, use it directly. Otherwise get the class from settings.
        """

        if self.serializer_class:
            return self.serializer_class
        try:
            return import_string(self._serializer_class)
        except ImportError:
            msg = "Could not import serializer '%s'" % self._serializer_class
            raise ImportError(msg)

    def get_queryset(self):
        queryset = Notification.objects.filter(recipient=self.request.user)

        if self.action in ("list", "count"):
            if notification_settings.get_config()['SOFT_DELETE']:
                queryset = queryset.active()
        elif self.action in ("unread", "unread_count", "mark_as_read", "mark_all_as_read"):
            queryset = queryset.unread()
        elif self.action in ("read", "read_count", "mark_as_unread"):
            queryset = queryset.read()

        return queryset
    
    @action(["get"], detail=False)
    def read(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @action(["get"], detail=False)
    def unread(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @action(["get"], detail=False)
    def read_count(self, request, *args, **kwargs):
        data = {'count': request.user.notifications.read().count()}
        return Response(data)

    @action(["get"], detail=False)
    def unread_count(self, request, *args, **kwargs):
        data = {'count': request.user.notifications.unread().count()}
        return Response(data)

    @action(["get"], detail=False)
    def count(self, request, *args, **kwargs):
        data = {'count': request.user.notifications.count()}
        return Response(data)

    @action(["put"], detail=True)
    def mark_as_read(self, request, *args, **kwargs):
        notification = self.get_object()
        notification.mark_as_read()
        return super().list(request, *args, **kwargs)

    @action(["put"], detail=True)
    def mark_as_unread(self, request, *args, **kwargs):
        notification = self.get_object()
        notification.mark_as_unread()
        return super().list(request, *args, **kwargs)

    @action(["put"], detail=False)
    def mark_all_as_read(self, request, *args, **kwargs):
        request.user.notifications.mark_all_as_read()
        return super().list(request, *args, **kwargs)
